import os
import numpy as np
import glob
import sys
import xarray as xr
import scipy
import pandas as pd
import ddeq

'''

PATHS

'''

ROOT = 'C:\\Users\\diego\\Desktop\\ddeq\\data\\'

EMISSIONS_DATA_PATH = os.path.join(ROOT, 'timeseries')
SMARTCARB_DATA_PATH = os.path.join(ROOT, 'level2')
WIND_DATA_PATH = os.path.join(ROOT, 'winds')

# list of point sources
sources = ddeq.misc.read_point_sources()
# sources = sources.sel(source=['Janschwalde'])
# No data for the true emissions of some sources
# We skip them
sources = sources.drop_sel(source='Hamburg')
sources = sources.drop_sel(source='Prague')
sources = sources.drop_sel(source='Belchatow')
sources = sources.drop_sel(source='Paris')
sources = sources.drop_sel(source='Lipetsk')
sources = sources.drop_sel(source='Matimba')


'''

    MAIN PARAMETERS OF THE ALGORITHM
    ALL PARAMETERS ARE DESCRIBED IN ddeq.lcs.estimate_emissions 

'''

lcs_params={}
lcs_params['max_estim_emis']=90
lcs_params['alti_ref']=200
lcs_params['wind_alti_avg']=False
lcs_params['wind_product']='ERA5' # ERA5 || SMARTCARB
lcs_params['wind_product']='SMARTCARB' # ERA5 || SMARTCARB
lcs_params['Enhancement_thresh']=1.
lcs_params['max_sigma_Gauss']=5.
lcs_params['window_width']=100
lcs_params['tracer_gases']=['NO2'] # ['CO2','NO2','CO2_with_NO2']
lcs_params['tracer_gases']=['CO2_with_NO2'] # ['CO2','NO2','CO2_with_NO2']
lcs_params['verbose']=True

#Reading l2 data without clouds: co2_cloud_threshold=1000 (>1)
co2_cloud_threshold=no2_cloud_threshold=1000 #No clouds
l2_filename='Sentinel_7_CO2_2015042311_o1670_l0483.nc'
l2_filename=os.path.join(SMARTCARB_DATA_PATH,l2_filename)
l2_data = ddeq.smartcarb.read_level2(l2_filename,co2_noise_scenario='medium',co2_cloud_threshold=co2_cloud_threshold,\
                                         no2_noise_scenario='high',no2_cloud_threshold=no2_cloud_threshold)

l2_data = ddeq.smartcarb.read_level2(l2_filename,co2_noise_scenario='medium',no2_noise_scenario='high')

#
#  EXTRACTING WINDS FIELD CORRESPONDING TO THE XCO2 data
#
obs_time= pd.Timestamp(l2_data.time.to_pandas())
obs_day=str(obs_time.year)+str(obs_time.month).zfill(2)+str(obs_time.day).zfill(2)

if lcs_params['wind_product']=='ERA5':
    if lcs_params['alti_ref']=='GNFR-A':
        wind_file='ERA5-gnfra-'+obs_day+'t'+str(obs_time.hour)+'00.nc'
        wind_alti_file=None
    else:
        wind_file='ERA5-lvl-'+obs_day+'t'+str(obs_time.hour)+'00.nc'
        wind_alti_file='ERA5-alti_lvl-'+obs_day+'t'+str(obs_time.hour)+'00.nc'

if lcs_params['wind_product']=='SMARTCARB':
    wind_file='SMARTCARB_winds_'+obs_day+str(obs_time.hour)+'.nc'
    wind_alti_file=None

wind_file=os.path.join(WIND_DATA_PATH,wind_file)
if wind_alti_file!=None:wind_alti_file=os.path.join(WIND_DATA_PATH,wind_alti_file)

wind_xr=ddeq.wind.get_wind_field(wind_file,wind_alti_file,lcs_params)

res_xr=ddeq.lcs.estimate_emissions(l2_data,wind_xr,sources,lcs_params,all_diags=True)

#
#Printing Results. Only works for SMARTCARB sources
#
ddeq.lcs.print_smartcarb_res(res_xr,sources,lcs_params,l2_data,EMISSIONS_DATA_PATH)

#
# Plotting results for a given source
#
plot_source='Janschwalde'
emi_time = pd.Timestamp(l2_data.time.values)
true_emis={}
for tracer_gas in ['CO2','NO2']:
    true_emis[tracer_gas]=ddeq.misc.read_true_emissions(emi_time,tracer_gas,plot_source,tmin=10,tmax=11,data_path=EMISSIONS_DATA_PATH).mean()

fig=ddeq.vis.plot_lcs_results(plot_source,res_xr,l2_data,sources,lcs_params,true_emis=true_emis)
fig.show()

